"""

:platform: Unix, Windows 
:last changed: 2023-01-19

.. moduleauthor:: Tjark Leon Raphael Gröne <tgroene@physnet.uni-hamburg.de>


This Code is used to import integrated .chi data and processes it to to the PDF .gr 
"""

from backbone.PDF_generation_class_Heat2 import PDF_generation_class, outputDatFile
import pathlib
rootdir = str(pathlib.Path(__file__).parent.resolve())
#----------------------------------------------------------------------------------------#
#                                                                                        #
#                                      Configurations                                    #
#                                                                                        #
#----------------------------------------------------------------------------------------#
# BG file name (directories for integrated data and metadata should be keept constant throughout the beamtime)
bg_sampleName = 'empty_ac_inlet_180_10' #filename
bg_path =  '/asap3/petra3/gpfs/p21.1/2023/data/11017732/processed/pe/ac_cell/' + bg_sampleName + "/" #integrated data path
bg_tempPath = '/asap3/petra3/gpfs/p21.1/2023/data/11017732/raw/pe/ac_cell/' + bg_sampleName + "/"  #metadat path

bg_Reload = None # faster loading after first full initiation (keep None during initiation)
bg_Reload = rootdir + '/processed/' + bg_sampleName + '/Reload_files/bg_Reload/' + bg_sampleName + '.mat'

# Just a Dummy as reference to trigger the last XRD scan to plot
references = {  'Dummy': {'path': rootdir + "/References/Dummy.int",
                            'skiprows': 2,
                            'energy': 100} ,}

ends_with = '.dat' # this can differe for differnet live integrations used, but should be the same for one beamtime

ramp_start_temp = 30  #set a little bit higher than baseline temperature from which the bg started
plato_start_temp = 180 #temperatur on which to stay in °C
average = 60 #number of scans to average over
Qmaxinst = 21.5 #Qmax at which the mask sets in 

check_phases_at_scan = 21 #choose last scan processed by the live integration devided by the number of averged scans




#----------------------------------------------------------------------------------------#
#                                                                                        #
#                                    Process data set                                    #
#                                                                                        #
#----------------------------------------------------------------------------------------#

#Loading the background data
startScan = 0
endScan = 1e10
bg = PDF_generation_class(path=bg_path, tempPath=bg_tempPath, sampleName=bg_sampleName, startScan=startScan, endScan=endScan, ends_with=ends_with)#, reload=bg_Reload)
bg.getTempPoints(ramp_start_temp,plato_start_temp)
bg_heat_ranges = {  "ramp1_start": bg.rampstart_index,
                    "ramp1_end": int(bg.platostart_index+100),
                    "ramp2_start": 100000,
                    "ramp2_end": 100000,}
endScan = bg.platoend_index
# bg = PDF_generation_class(path=bg_path, tempPath=bg_tempPath, sampleName=bg_sampleName, startScan=startScan, endScan=endScan, ends_with=ends_with, reload=bg_Reload)
bg.saveReload(rootdir + '/processed/' + bg_sampleName + '/Reload_files/bg_Reload/')

bg.useAverage(average)
bg.norm(Qmaxinst=Qmaxinst)
bg.outputDatFile(rootdir + '/processed/' + bg_sampleName + '/Dat_files/bg_Dat/')

bg.getRampPlatoLists(bg_heat_ranges)
bg_plato_list = bg.plato_list
bg_ramp_list = bg.ramp_list

bg.baselineCorrection(40)
bg.Filter('whittaker',20)
bg.checkPhases(check_phases_at_scan,references=references,savefig='png',outputPath=rootdir + '/processed/' + bg_sampleName + '/XRD_Plots/bg_phase_check/', maxQ=9)
bg.plot2dTemp(savefig='svg',outputPath=rootdir + '/processed/' + bg_sampleName + '/XRD_Plots/bg_2D_plot/')
bg.plot1dvsTime(step=2,maxQ=15, max_index=check_phases_at_scan, plot_temp=True,savefig='svg',outputPath=rootdir + '/processed/' + bg_sampleName + '/XRD_Plots/bg_1D_plot/')

bg.PrintLogFile(rootdir + '/processed/' + bg_sampleName + '/Log_files/bg_Log/')
#-----------------------------------------------------------------------------------------#
