"""

:platform: Unix, Windows 
:last changed: 2022-05-14

.. moduleauthor:: Tjark Leon Raphael Gröne <tgroene@physnet.uni-hamburg.de>

This code is used to load and plot pdf data
-- There for exitu data we will load the class called exituPDF
-- To display insitu data the insituPDF class is suitable
-- Last there is the insituPDFfitted class you can use to display 
    the different properties that the diffpy-cmi will print in the result files

"""

import backbone.PDF_class_insitu as PDF_class
import pathlib
rootdir = str(pathlib.Path(__file__).parent.resolve())


#----------------------------------------------------------------------------------------#
#                                                                                        #
#                                      Configurations                                    #
#                                                                                        #
#----------------------------------------------------------------------------------------#

startScan = 1       #Define the first fscan to process for the plot
endScan = 10000        #Define the last fscan to process for the plot
average = 120      #Averaging as used in the PDF generation
step_width = 6     #To shortan the number of scans in a 1D plot step lenght can be increased


figratio1D =[4,3]   #Define the window size ratio for a 1D plot
figratio2D =[7,4]   #Define the window size ratio for a 2D plot
#----------------------------------------------------------------------------------------#
#                                                                                        #
#                                    Process data set                                    #
#                                           1                                            #
#----------------------------------------------------------------------------------------#


# Here we call our data set, which we want to look at. --> Needs to undergo the PDF generation first ;)
sample_name = 'fc_zns_1cm_3' #filename
# sample_name = 'zns_2p_10_2' #filename

path_to_Temp = rootdir + "/processed/" + sample_name + "/Reload_files/meas_Reload/fc_zns_1cm_3.mat"
# path_to_Temp = rootdir + "/processed/" + sample_name + "/Reload_files/meas_Reload/zns_2p_10_2.mat"


# Here we can choose which type of PDF generation we want to look at. 
path =  rootdir + "/processed/" + sample_name + "/PDF_files/meas_type1_uncorrected_PDF/" 

# Here the everthing is transfered to the class used for plotting the PDF:
# One can flip (y = -y) certain scans by using -- , reverse_data=[1,7], reverse_style="list") -- if nessicary
meas = PDF_class.InsituPDF(path=path, sampleName=sample_name, startScan=startScan, endScan=endScan,
                             timeOfset=0, tempPath=path_to_Temp, average=average,
                            figratio1D=figratio1D,figratio2D=figratio2D)

# You can also save the loaded .dat files in a .mat reload file for faster reloading (often I just reprocess)
# meas.saveReload(rootdir + "processed/Reload") 

# Waterfall like 1D plot (each scan is offset by the intercept value and color coded to the time scale)
# meas.plot1dvsTimeProjectedOntop(offsetTime=0,savefig="svg",outputPath=rootdir + "/processed/" + sample_name + "//PDF_Plots/PDF_1D_projected_plot/",jump=step_width, minR=1.65, maxR=10, intercept=0.2) #plot in 1D, insitu=True will define color change and hides ledgend

# # 1D plot over time with temperature label (if many scans are take and the averaging is low^and you still want to use a small step width the labels can be removed)
meas.plot1dvsTime(offsetTime=0,savefig="svg",outputPath=rootdir + "/processed/" + sample_name + "/PDF_Plots/PDF_1D_plot/",step_width=step_width, minR=1, maxR=15) #plot in 1D, insitu=True will define color change and hides ledgend

# 2D plot over time with temperature curve
meas.plot2dTemp(vmin=-1.5,vmax=5,savefig="svg",outputPath= rootdir + '/processed/' + sample_name + '/PDF_Plots/PDF_2D_plot/', maxR=15) #rect=rectangle)
