from backbone.functions.utils import ensure_dir, del_folder_content
from time import time
from scipy import io as sp_io 
import numpy as np
import os



class Export_PDF_Plot:
    """ Class for exporting PDF data to .mat and temperature and time to .xy
    """
    #----------------------------------------------------------------------------------------#
    #                              save data for reloading                                   #
    #----------------------------------------------------------------------------------------#
    def saveReload(self,outputPath):
        """ Saves PDF data do a .mat file.
        
        Arguments
        ----------
        outputPath: Path to output directory
        """    
        
        #Generate directory if not existing
        if outputPath != None:
            ensure_dir(outputPath)
            
        # Save the data
        sp_io.savemat(outputPath + "/" + self.sampleName +'.mat', {'G_of_r': self.G, 'r': self.r,'time': self.time_array ,'time_averaged': self.time_averaged_array,'temp': self.temp_array, 'temp_averaged':self.temp_averaged_array})


        # Getting feedback: Not solved nicely I kwon, but it works 
        print('\n ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~')
        print(f'           Reload files created for {self.sampleName}, fast loading possible')
        print(' ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n')

    #----------------------------------------------------------------------------------------#
    #                                  export temp as xy                                     #
    #----------------------------------------------------------------------------------------#
    def outputTemp(self,outputPath):
        """ Save time and temperature to .xy
        Arguments
        ----------
        outputPath: Path to output directory
        """
        
        #Generate directory if not existing
        if outputPath != None:
            ensure_dir(outputPath)
            
        #Data output file
        data = np.array([self.time_array, self.temp_array])
        outfname = outputPath + "/" + self.path.split("/")[-1].split(".")[0]+ '.xy' #output directory and filename
        np.savetxt(outfname, data.T,header = 'Time(s) Temperature(°C)')



    
class Export_PDF_Gen:
    """ Exports XRD/TS data and metadata to .mat/.dat and .log
    """
            
    #----------------------------------------------------------------------------------------#
    #                              save data for reloading                                   #
    #----------------------------------------------------------------------------------------#
    def saveReload(self,outputPath):
        """ Saves XRD/TS data do a .mat file.
        Arguments
        ----------
        outputPath: Path to output directory
        """    
        
        # Log the event
        self.logger['Reload .mat was created'] = {   'Output Path': outputPath,}
        
        # Generate directory if not existing
        if outputPath != None:
            ensure_dir(outputPath)
            
        # Save the data
        sp_io.savemat(outputPath + "/" + self.sampleName +'.mat', {'I_matrix': self.I, 'q_axis': self.q,'time': self.time_array ,'time_averaged': self.time_average_array,'temp': self.temp_array, 'temp_averaged':self.temp_average_array})
        return self.logger

    #----------------------------------------------------------------------------------------#
    #                                  Export .mat for MCR-ALS                                #
    #----------------------------------------------------------------------------------------#
    def outputMatFile(self,outputPath):
        """ Saves only unaltered XRD/TS data do a .mat file.
        Arguments
        ----------
        outputPath: Path to output directory
        """   
        
        # Log the event 
        self.logger['Data was stored in .mat'] = {   'Output Path': outputPath,}
        self.logger['Mat file saved'] = {'Output path': outputPath + "/" + self.sampleName +'.mat'}
        
        # Generate directory if not existing
        if outputPath != None:
            ensure_dir(outputPath)
            
        # Save the data
        sp_io.savemat(outputPath + "/" + self.sampleName +'.mat', {'I_matrix': self.I_original, 'q_axis': self.q,'I_processed':self.I ,'time': self.time_array ,'time_averaged': self.time_averaged_array,'temp': self.temp_array, 'temp_averaged':self.temp_averaged_array})
        return self.logger

    #----------------------------------------------------------------------------------------#
    #                                   Export .dat to export                                #
    #----------------------------------------------------------------------------------------#
    def outputDatFile(self,outputPath):
        """ Saves XRD/TS data do a .dat file.
        Arguments
        ----------
        outputPath: Path to output directory
        """
        
        # Log the event 
        self.logger['Data was saved in .dat for ever scan'] = {'Output Path': outputPath,}
        
        # Delete old content in directory or generate directory if not existing
        if outputPath != None:
            if os.path.exists(outputPath):
                del_folder_content(outputPath)
            ensure_dir(outputPath)

        # Data output file
        for i, j in enumerate(self.I):
            data = np.array([self.q, j])
            outfname = outputPath + "/" + self.sampleName + "_" + str((i+1)*self.average + self.startScan).zfill(5) + '.txt' #output directory and filename
            np.savetxt(outfname, data.T,header = 'q (Angstrom), Intensity (a.u.)')
        return self.logger

    #----------------------------------------------------------------------------------------#
    #                                Output the loged actions                                #
    #----------------------------------------------------------------------------------------#
    def PrintLogFile(self,outputPath):
        """ Saves all events happend to .log file.
        
        Arguments
        ----------
        outputPath: Destination of the .log file
        """
        
        # Generate directory if not existing
        if outputPath != None:
            ensure_dir(outputPath)
            
        # Here we work in a appending style so the .log file can be reopend and added to. 
        log_file = open(outputPath + '/' + self.sampleName + ".log", "w")
        for x in self.logger:
            log_file.write('\n' + str(x) + '\n')
            for y in self.logger[x]:
                log_file.write(str(y) + '  :  ' + str(self.logger[x][y]) +'\n')
        log_file.close()

    
#----------------------------------------------------------------------------------------#
#                             Output Dat from BG for PDFgetX3                            #
#----------------------------------------------------------------------------------------#
def outputDatFile(outputPath,x,matrix,average,sampleName,clear_folder: bool=False):
    """ Generating a .dat file from the given data arrays.

    Args:
        outputPath (str): Path to output directory.
        x (list, array): x-Axis array of data points.
        matrix (list, array): y-Axis matrix of scans with y data points.
        average (int): Degree to which scans are averaged.
        sampleName (str): Name of the sample.
        clear_folder (bool, optional): Delete folder content before saving new files. Defaults to False.
    """

    # Delete old content in directory or generate directory if not existing
    if outputPath != None:
        if os.path.exists(outputPath) and clear_folder==True:
            del_folder_content(outputPath)
        ensure_dir(outputPath)
    
    #Data output file
    for i, j in enumerate(matrix):
        data = np.array([x, j])
        outfname = outputPath + "/" + sampleName + "_" + str((i+1)*average).zfill(5) + '.txt' #output directory and filename
        np.savetxt(outfname, data.T,header = 'q (Angstrom), Intensity (a.u.)')